@@ -3,12 +3,13 @@
 // Purpose:     
 // Author:      Alex Thuering
 // Created:     2005/05/09
-// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.26 2011/06/05 19:30:28 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.31 2011/07/24 17:15:32 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 //////////////////////////////////////////////////////////////////////////////
 
 #include <wx/wx.h>
+#include <wx/tokenzr.h>
 #include "SVGCanvasItem.h"
 #include "SVGCanvas.h"
 #include <math.h>
@@ -526,176 +527,158 @@ bool wxSVGCanvasPath::ClosePath()
 WX_DEFINE_OBJARRAY(wxSVGCanvasTextCharList);
 WX_DEFINE_OBJARRAY(wxSVGCanvasTextChunkList);
 
-wxSVGCanvasText::wxSVGCanvasText(wxSVGCanvas* canvas):
-  wxSVGCanvasItem(wxSVG_CANVAS_ITEM_TEXT), m_canvas(canvas)
-{
-  m_char = NULL;
-  m_tx = m_ty = 0;
-  m_textAnchor = wxCSS_VALUE_START;
-  m_dominantBaseline = wxCSS_VALUE_AUTO;
+wxSVGCanvasText::wxSVGCanvasText(wxSVGCanvas* canvas) :
+	wxSVGCanvasItem(wxSVG_CANVAS_ITEM_TEXT), m_canvas(canvas) {
+	m_char = NULL;
+	m_tx = m_ty = 0;
+	m_textAnchor = wxCSS_VALUE_START;
+	m_dominantBaseline = wxCSS_VALUE_AUTO;
 }
 
-wxSVGCanvasText::~wxSVGCanvasText()
-{
-  for (unsigned int i=0; i<m_chunks.Count(); i++)
-  	for (unsigned int j=0; j<m_chunks[i].chars.Count(); j++)
-		delete m_chunks[i].chars[j].path;
+wxSVGCanvasText::~wxSVGCanvasText() {
+	for (unsigned int i = 0; i < m_chunks.Count(); i++)
+		for (unsigned int j = 0; j < m_chunks[i].chars.Count(); j++)
+			delete m_chunks[i].chars[j].path;
 }
 
-void wxSVGCanvasText::Init(wxSVGTextElement& element,
-  const wxCSSStyleDeclaration& style)
-{
-  m_tx = element.GetX().GetAnimVal().Count() ? element.GetX().GetAnimVal()[0] : wxSVGLength(0);
-  m_ty = element.GetY().GetAnimVal().Count() ? element.GetY().GetAnimVal()[0] : wxSVGLength(0);
-  InitChildren(element, style);
-  EndTextAnchor();
-}
-
-void wxSVGCanvasText::Init(wxSVGTSpanElement& element,
-  const wxCSSStyleDeclaration& style)
-{
-  if (element.GetX().GetAnimVal().Count())
-	EndTextAnchor();
-  
-  if (element.GetX().GetAnimVal().Count())
-	m_tx = element.GetX().GetAnimVal()[0];
-  if (element.GetY().GetAnimVal().Count())
-	m_ty = element.GetY().GetAnimVal()[0];
-  InitChildren(element, style);
-  
-  if (element.GetX().GetAnimVal().Count())
+void wxSVGCanvasText::Init(wxSVGTextElement& element, const wxCSSStyleDeclaration& style) {
+	m_tx = element.GetX().GetAnimVal().Count() ? element.GetX().GetAnimVal()[0] : wxSVGLength(0);
+	m_ty = element.GetY().GetAnimVal().Count() ? element.GetY().GetAnimVal()[0] : wxSVGLength(0);
+	InitChildren(element, style);
 	EndTextAnchor();
 }
 
-void wxSVGCanvasText::InitChildren(wxSVGTextPositioningElement& element,
-  const wxCSSStyleDeclaration& style)
-{
-  wxSVGElement* elem = (wxSVGElement*) element.GetChildren();
-  while (elem)
-  {
-	if (elem->GetType() == wxSVGXML_TEXT_NODE)
-	{
-	  BeginChunk(style);
-	  InitText(elem->GetContent(), style);
-	}
-	if (elem->GetType() == wxSVGXML_ELEMENT_NODE &&
-		elem->GetDtd() == wxSVG_TSPAN_ELEMENT)
-	{
-	  wxSVGTSpanElement& tElem = (wxSVGTSpanElement&)*elem;
-	  wxCSSStyleDeclaration tStyle(style);
-	  tStyle.Add(tElem.GetStyle());
-	  Init(tElem, tStyle);
-	}
-	elem = (wxSVGElement*) elem->GetNext();
-  }
+void wxSVGCanvasText::Init(wxSVGTSpanElement& element, const wxCSSStyleDeclaration& style) {
+	if (element.GetX().GetAnimVal().Count())
+		EndTextAnchor();
+	
+	if (element.GetX().GetAnimVal().Count())
+		m_tx = element.GetX().GetAnimVal()[0];
+	if (element.GetY().GetAnimVal().Count())
+		m_ty = element.GetY().GetAnimVal()[0];
+	InitChildren(element, style);
+	
+	if (element.GetX().GetAnimVal().Count())
+		EndTextAnchor();
 }
 
-void wxSVGCanvasText::BeginChunk(const wxCSSStyleDeclaration& style)
-{
-  wxSVGCanvasTextChunk* chunk = new wxSVGCanvasTextChunk;
-  chunk->style.Add(style);
-  m_chunks.Add(chunk);
-  
-  if (style.HasTextAnchor() && m_textAnchor == wxCSS_VALUE_START)
-  {
-	m_textAnchor = style.GetTextAnchor();
-	m_textAnchorBeginIndex = m_chunks.Count()-1;
-	// save x-pos: if text anchor is wxCSS_VALUE_END, we will continue from this point
-	m_textAnchorBeginPos = m_tx;
-  }
-  if (style.HasDominantBaseline() && (m_dominantBaseline == wxCSS_VALUE_AUTO
-  		|| m_dominantBaseline == wxCSS_VALUE_ALPHABETIC))
-  {
-	m_dominantBaseline = style.GetDominantBaseline();
-	m_dominantBaselineBeginIndex = m_chunks.Count()-1;
-  }
+void wxSVGCanvasText::InitChildren(wxSVGTextPositioningElement& element, const wxCSSStyleDeclaration& style) {
+	wxString text;
+	wxSVGElement* elem = (wxSVGElement*) element.GetChildren();
+	while (elem) {
+		if (elem->GetType() == wxSVGXML_TEXT_NODE) {
+			wxStringTokenizer tokenizer(elem->GetContent());
+			while (tokenizer.HasMoreTokens()) {
+				text += tokenizer.GetNextToken();
+				if (tokenizer.HasMoreTokens())
+					text += wxT(" ");
+			}
+		} else if (elem->GetType() == wxSVGXML_ELEMENT_NODE && elem->GetDtd() == wxSVG_TBREAK_ELEMENT)
+			text += wxT("\n");
+		else if (text.length())
+			AddChunk(text, style);
+		
+		if (elem->GetType() == wxSVGXML_ELEMENT_NODE && elem->GetDtd() == wxSVG_TSPAN_ELEMENT) {
+			wxSVGTSpanElement& tElem = (wxSVGTSpanElement&) *elem;
+			wxCSSStyleDeclaration tStyle(style);
+			tStyle.Add(tElem.GetStyle());
+			Init(tElem, tStyle);
+		}
+		elem = (wxSVGElement*) elem->GetNext();
+	}
+	if (text.length())
+		AddChunk(text, style);
 }
 
-void wxSVGCanvasText::BeginChar()
-{
+void wxSVGCanvasText::AddChunk(const wxString& text, const wxCSSStyleDeclaration& style) {
+	wxSVGCanvasTextChunk* chunk = new wxSVGCanvasTextChunk;
+	chunk->style.Add(style);
+	chunk->x = m_tx;
+	chunk->y = m_ty;
+	chunk->text = text;
+	m_chunks.Add(chunk);
+	
+	// set textAnchor and dominantBaseline if they are not already set
+	if (style.HasTextAnchor() && m_textAnchor == wxCSS_VALUE_START) {
+		m_textAnchor = style.GetTextAnchor();
+		m_textAnchorBeginIndex = m_chunks.Count() - 1;
+		m_textAnchorBeginPos = m_tx;
+	}
+	if (style.HasDominantBaseline() && (m_dominantBaseline == wxCSS_VALUE_AUTO || m_dominantBaseline
+			== wxCSS_VALUE_ALPHABETIC)) {
+		m_dominantBaseline = style.GetDominantBaseline();
+		m_dominantBaselineBeginIndex = m_chunks.Count() - 1;
+	}
+	
+	InitText(text, style);
+}
+
+void wxSVGCanvasText::BeginChar() {
 	m_char = new wxSVGCanvasTextChar;
 	m_char->path = m_canvas->CreateCanvasPath();
-	m_chunks[m_chunks.GetCount()-1].chars.Add(m_char);
+	m_chunks[m_chunks.GetCount() - 1].chars.Add(m_char);
 }
 
-void wxSVGCanvasText::EndChar()
-{
+void wxSVGCanvasText::EndChar() {
 	m_char->path->End();
 }
 
-void wxSVGCanvasText::EndTextAnchor()
-{
-  if (m_textAnchor != wxCSS_VALUE_START)
-  {
-	for (int i=m_textAnchorBeginIndex; i<(int)m_chunks.Count(); i++)
-	{
-	  wxSVGCanvasTextChunk& chunk = m_chunks[i];
-	  if (m_textAnchor == wxCSS_VALUE_END)
-		chunk.matrix = chunk.matrix.Translate(m_textAnchorBeginPos-m_tx, 0);
-	  else if (m_textAnchor == wxCSS_VALUE_MIDDLE)
-		chunk.matrix = chunk.matrix.Translate((m_textAnchorBeginPos-m_tx)/2, 0);
+void wxSVGCanvasText::EndTextAnchor() {
+	if (m_textAnchor != wxCSS_VALUE_START) {
+		for (int i = m_textAnchorBeginIndex; i < (int) m_chunks.Count(); i++) {
+			wxSVGCanvasTextChunk& chunk = m_chunks[i];
+			if (m_textAnchor == wxCSS_VALUE_END)
+				chunk.matrix = chunk.matrix.Translate(m_textAnchorBeginPos - m_tx, 0);
+			else if (m_textAnchor == wxCSS_VALUE_MIDDLE)
+				chunk.matrix = chunk.matrix.Translate((m_textAnchorBeginPos - m_tx) / 2, 0);
+		}
+		if (m_textAnchor == wxCSS_VALUE_END)
+			m_tx = m_textAnchorBeginPos;
+		else if (m_textAnchor == wxCSS_VALUE_MIDDLE)
+			m_tx -= (m_textAnchorBeginPos - m_tx) / 2;
+		m_textAnchor = wxCSS_VALUE_START;
 	}
-	if (m_textAnchor == wxCSS_VALUE_END)
-	  m_tx = m_textAnchorBeginPos;
-	else if (m_textAnchor == wxCSS_VALUE_MIDDLE)
-	  m_tx -= (m_textAnchorBeginPos-m_tx)/2;
-	m_textAnchor = wxCSS_VALUE_START;
-  }
-  if (m_dominantBaseline != wxCSS_VALUE_AUTO
-  		&& m_dominantBaseline != wxCSS_VALUE_ALPHABETIC)
-  {
-	for (int i=m_dominantBaselineBeginIndex; i<(int)m_chunks.Count(); i++)
-	{
-		wxSVGCanvasTextChunk& chunk = m_chunks[i];
-		wxSVGRect chunkBBox = chunk.GetBBox();
-		if (chunkBBox.IsEmpty())
-			continue;
-		if (m_dominantBaseline == wxCSS_VALUE_MIDDLE
-	  			|| m_dominantBaseline == wxCSS_VALUE_CENTRAL)
-			chunk.matrix = chunk.matrix.Translate(0,
-					m_ty - chunkBBox.GetY() - chunkBBox.GetHeight()/2);
-		else if (m_dominantBaseline == wxCSS_VALUE_TEXT_AFTER_EDGE)
-			chunk.matrix = chunk.matrix.Translate(0,
-					m_ty - chunkBBox.GetY());
-		else if (m_dominantBaseline == wxCSS_VALUE_TEXT_BEFORE_EDGE)
-			chunk.matrix = chunk.matrix.Translate(0,
-					m_ty - chunkBBox.GetY() - chunkBBox.GetHeight());
+	if (m_dominantBaseline != wxCSS_VALUE_AUTO && m_dominantBaseline != wxCSS_VALUE_ALPHABETIC) {
+		for (int i = m_dominantBaselineBeginIndex; i < (int) m_chunks.Count(); i++) {
+			wxSVGCanvasTextChunk& chunk = m_chunks[i];
+			wxSVGRect chunkBBox = chunk.GetBBox();
+			if (chunkBBox.IsEmpty())
+				continue;
+			if (m_dominantBaseline == wxCSS_VALUE_MIDDLE || m_dominantBaseline == wxCSS_VALUE_CENTRAL)
+				chunk.matrix = chunk.matrix.Translate(0, m_ty - chunkBBox.GetY() - chunkBBox.GetHeight() / 2);
+			else if (m_dominantBaseline == wxCSS_VALUE_TEXT_AFTER_EDGE)
+				chunk.matrix = chunk.matrix.Translate(0, m_ty - chunkBBox.GetY());
+			else if (m_dominantBaseline == wxCSS_VALUE_TEXT_BEFORE_EDGE)
+				chunk.matrix = chunk.matrix.Translate(0, m_ty - chunkBBox.GetY() - chunkBBox.GetHeight());
+		}
+		m_dominantBaseline = wxCSS_VALUE_AUTO;
 	}
-	m_dominantBaseline = wxCSS_VALUE_AUTO;
-  }
 }
 
-extern wxSVGRect TransformRect(wxSVGRect rect, wxSVGMatrix& matrix);
-
-wxSVGRect wxSVGCanvasTextChunk::GetBBox(const wxSVGMatrix& matrix)
-{
-  wxSVGRect bbox;
-  for (int i=0; i<(int)chars.Count(); i++)
-  {
-    wxSVGRect elemBBox = chars[i].path->GetBBox(matrix);
-    if (elemBBox.IsEmpty())
-      elemBBox = &matrix ? chars[i].bbox.MatrixTransform(matrix) : chars[i].bbox;
-	if (i == 0)
-	  bbox = elemBBox;
-	else
-	{
-	  if (bbox.GetX() > elemBBox.GetX())
-	  {
-		bbox.SetWidth(bbox.GetWidth() + bbox.GetX() - elemBBox.GetX());
-		bbox.SetX(elemBBox.GetX());
-	  }
-	  if (bbox.GetY() > elemBBox.GetY())
-	  {
-		bbox.SetHeight(bbox.GetHeight() + bbox.GetY() - elemBBox.GetY());
-		bbox.SetY(elemBBox.GetY());
-	  }
-	  if (bbox.GetX() + bbox.GetWidth() < elemBBox.GetX() + elemBBox.GetWidth())
-		bbox.SetWidth(elemBBox.GetX() + elemBBox.GetWidth() - bbox.GetX());
-	  if (bbox.GetY() + bbox.GetHeight() < elemBBox.GetY() + elemBBox.GetHeight())
-		bbox.SetHeight(elemBBox.GetY() + elemBBox.GetHeight() - bbox.GetY());
+wxSVGRect wxSVGCanvasTextChunk::GetBBox(const wxSVGMatrix& matrix) {
+	wxSVGRect bbox;
+	for (int i = 0; i < (int) chars.Count(); i++) {
+		wxSVGRect elemBBox = chars[i].path->GetBBox(matrix);
+		if (elemBBox.IsEmpty())
+			elemBBox = &matrix ? chars[i].bbox.MatrixTransform(matrix) : chars[i].bbox;
+		if (i == 0)
+			bbox = elemBBox;
+		else {
+			if (bbox.GetX() > elemBBox.GetX()) {
+				bbox.SetWidth(bbox.GetWidth() + bbox.GetX() - elemBBox.GetX());
+				bbox.SetX(elemBBox.GetX());
+			}
+			if (bbox.GetY() > elemBBox.GetY()) {
+				bbox.SetHeight(bbox.GetHeight() + bbox.GetY() - elemBBox.GetY());
+				bbox.SetY(elemBBox.GetY());
+			}
+			if (bbox.GetX() + bbox.GetWidth() < elemBBox.GetX() + elemBBox.GetWidth())
+				bbox.SetWidth(elemBBox.GetX() + elemBBox.GetWidth() - bbox.GetX());
+			if (bbox.GetY() + bbox.GetHeight() < elemBBox.GetY() + elemBBox.GetHeight())
+				bbox.SetHeight(elemBBox.GetY() + elemBBox.GetHeight() - bbox.GetY());
+		}
 	}
-  }
-  return bbox;
+	return bbox;
 }
 
 wxSVGRect wxSVGCanvasText::GetBBox(const wxSVGMatrix& matrix)
@@ -738,7 +721,7 @@ long wxSVGCanvasText::GetNumberOfChars()
 	return res;
 }
 
-wxSVGCanvasTextChunk* wxSVGCanvasText::getChunk(unsigned long& charnum)
+wxSVGCanvasTextChunk* wxSVGCanvasText::GetChunk(unsigned long& charnum)
 {
 	for (int i=0; i<(int)m_chunks.Count(); i++)
 	{
@@ -771,8 +754,8 @@ double wxSVGCanvasText::GetComputedTextLength()
 double wxSVGCanvasText::GetSubStringLength(unsigned long charnum, unsigned long nchars)
 {
 	unsigned long lastCharnum = charnum + nchars - 1;
-	wxSVGCanvasTextChunk* firstChunk = getChunk(charnum);
-	wxSVGCanvasTextChunk* lastChunk = getChunk(lastCharnum);
+	wxSVGCanvasTextChunk* firstChunk = GetChunk(charnum);
+	wxSVGCanvasTextChunk* lastChunk = GetChunk(lastCharnum);
 	if (firstChunk != NULL && lastChunk != NULL)
 	{
 		wxSVGRect bboxFirst = firstChunk->chars[charnum].path->GetBBox();
@@ -790,7 +773,7 @@ double wxSVGCanvasText::GetSubStringLength(unsigned long charnum, unsigned long
 
 wxSVGPoint wxSVGCanvasText::GetStartPositionOfChar(unsigned long charnum)
 {
-	wxSVGCanvasTextChunk* chunk = getChunk(charnum);
+	wxSVGCanvasTextChunk* chunk = GetChunk(charnum);
 	if (chunk != NULL)
 	{
 		wxSVGRect bbox = chunk->chars[charnum].path->GetBBox();
@@ -804,7 +787,7 @@ wxSVGPoint wxSVGCanvasText::GetStartPositionOfChar(unsigned long charnum)
 
 wxSVGPoint wxSVGCanvasText::GetEndPositionOfChar(unsigned long charnum)
 {
-    wxSVGCanvasTextChunk* chunk = getChunk(charnum);
+    wxSVGCanvasTextChunk* chunk = GetChunk(charnum);
 	if (chunk != NULL)
 	{
 		wxSVGRect bbox = chunk->chars[charnum].path->GetBBox();
@@ -818,7 +801,7 @@ wxSVGPoint wxSVGCanvasText::GetEndPositionOfChar(unsigned long charnum)
 
 wxSVGRect wxSVGCanvasText::GetExtentOfChar(unsigned long charnum)
 {
-    wxSVGCanvasTextChunk* chunk = getChunk(charnum);
+    wxSVGCanvasTextChunk* chunk = GetChunk(charnum);
 	if (chunk != NULL)
 	{
 		wxSVGRect bbox = chunk->chars[charnum].path->GetBBox();
@@ -861,7 +844,7 @@ double wxSVGCanvasText::GetRotationOfChar(unsigned long charnum)
 ////////////////////////////// wxSVGCanvasImage //////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-void wxSVGCanvasImage::Init(wxSVGImageElement& element) {
+void wxSVGCanvasImage::Init(wxSVGImageElement& element, const wxCSSStyleDeclaration& style) {
 	m_x = element.GetX().GetAnimVal();
 	m_y = element.GetY().GetAnimVal();
 	m_width = element.GetWidth().GetAnimVal();
@@ -892,21 +875,22 @@ void wxSVGCanvasImage::Init(wxSVGImageElement& element) {
 			if (decoder.Load(filename)) {
 				double duration = decoder.GetDuration();
 				if (duration > 0 || pos > 0) {
-					double dpos = pos > 0 ? ((double)pos)/1000 : duration * 0.05;
-					if (!decoder.SetPosition(dpos > 0.5 ? dpos - 0.5 : dpos)) {
+					m_image = decoder.GetNextFrame();
+					double dpos = pos > 0 ? ((double)pos)/1000 : (duration < 6000 ? duration * 0.05 : 300);
+					if (!decoder.SetPosition(dpos > 1.0 ? dpos - 1.0 : 0.0)) {
 						wxLog* oldLog = wxLog::SetActiveTarget(new wxLogStderr());
 						wxLogError(wxT("decoder.GetDuration(): %f"), duration);
 						wxLogError(wxT("decoder.SetPosition(%f) failed"), dpos > 0.5 ? dpos - 0.5 : dpos);
 						delete wxLog::SetActiveTarget(oldLog);
 					}
-					for (int i = 0; i < 100; i++) {
+					for (int i = 0; i < 60; i++) {
 						m_image = decoder.GetNextFrame();
 						double dpos1 = decoder.GetPosition();
 						if (dpos1 >= dpos || dpos1 < 0)
 							break;
 					}
 				} else {
-					for (int i = 0; i < 15; i++)
+					for (int i = 0; i < 30; i++)
 						m_image = decoder.GetNextFrame();
 				}
 				if (m_image.Ok() && decoder.GetFrameAspectRatio() > 0)
@@ -939,8 +923,7 @@ wxSVGCanvasVideo::~wxSVGCanvasVideo()
 #endif
 }
 
-void wxSVGCanvasVideo::Init(wxSVGVideoElement& element)
-{
+void wxSVGCanvasVideo::Init(wxSVGVideoElement& element, const wxCSSStyleDeclaration& style) {
   m_x = element.GetX().GetAnimVal();
   m_y = element.GetY().GetAnimVal();
   m_width = element.GetWidth().GetAnimVal();
