@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Decoder
 // Author:      Alex Thuering
 // Created:     21.07.2007
-// RCS-ID:      $Id: mediadec_ffmpeg.cpp,v 1.11 2011/02/23 19:25:24 ntalex Exp $
+// RCS-ID:      $Id: mediadec_ffmpeg.cpp,v 1.14 2011/07/22 20:57:15 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -46,34 +46,23 @@ void wxFfmpegMediaDecoder::Init() {
 }
 
 void PrintError(const wxString& msg, int err) {
-	switch (err) {
-	case AVERROR_INVALIDDATA:
-		wxLogError(wxT("%s: Error while parsing header"), msg.c_str());
-		break;
-	case AVERROR_NOFMT:
-		wxLogError(wxT("%s: Unknown format"), msg.c_str());
-		break;
-	case AVERROR(EIO):
-		wxLogError(wxT("%s: I/O error occured\n"
-				"Usually that means that input file is truncated and/or corrupted."), msg.c_str());
-		break;
-	case AVERROR(ENOMEM):
-		wxLogError(wxT("%s: memory allocation error occured"), msg.c_str());
-		break;
-	case AVERROR(ENOENT):
-		wxLogError(wxT("%s: no such file or directory"), msg.c_str());
-		break;
-	default:
-		wxLogError(wxT("%s: Error while opening file"), msg.c_str());
-		break;
-	}
+	char errbuf[128];
+	const char *errbuf_ptr = errbuf;
+	
+	if (av_strerror(err, errbuf, sizeof(errbuf)) < 0)
+		errbuf_ptr = strerror(AVUNERROR(err));
+	wxString errorStr(errbuf_ptr, wxConvUTF8);
+	wxLogError(msg + wxT(": ") + errorStr);
 }
 
-
 bool wxFfmpegMediaDecoder::Load(const wxString& fileName) {
 	Close();
+	if (!wxFileExists(fileName)) {
+		wxLogError(wxT("%s: no such file or directory"), fileName.c_str());
+		return false;
+	}
 	int err = av_open_input_file(&m_formatCtx, fileName.mb_str(), NULL, 0, NULL);
-	if (err !=0) {
+	if (err != 0) {
 		PrintError(fileName, err);
 		return false;
 	}
@@ -106,7 +95,7 @@ float wxFfmpegMediaDecoder::GetFrameAspectRatio() {
 	for (int i=0; i<(int)m_formatCtx->nb_streams; i++) {
 		AVStream *st = m_formatCtx->streams[i];
 		AVCodecContext *enc = st->codec;
-		if (enc->codec_type == CODEC_TYPE_VIDEO) {
+		if (enc->codec_type == AVMEDIA_TYPE_VIDEO) {
 			if (st->sample_aspect_ratio.num)
 				frame_aspect_ratio = av_q2d(st->sample_aspect_ratio);
 			else if (enc->sample_aspect_ratio.num)
@@ -124,11 +113,11 @@ StreamType wxFfmpegMediaDecoder::GetStreamType(unsigned int streamIndex) {
 	if (m_formatCtx == NULL || streamIndex >= m_formatCtx->nb_streams)
 		return stUNKNOWN;
 	switch (m_formatCtx->streams[streamIndex]->codec->codec_type) {
-	case CODEC_TYPE_VIDEO:
+	case AVMEDIA_TYPE_VIDEO:
 		return stVIDEO;
-	case CODEC_TYPE_AUDIO:
+	case AVMEDIA_TYPE_AUDIO:
 		return stAUDIO;
-	case CODEC_TYPE_SUBTITLE:
+	case AVMEDIA_TYPE_SUBTITLE:
 		return stSUBTITLE;
 	default:
 		break;
@@ -167,7 +156,7 @@ bool wxFfmpegMediaDecoder::OpenVideoDecoder() {
 	// find the first video stream
 	m_videoStream = -1;
 	for (int i=0; i<(int)m_formatCtx->nb_streams; i++) {
-		if (m_formatCtx->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO) {
+		if (m_formatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO) {
 			m_videoStream = i;
 			break;
 		}
@@ -246,42 +235,34 @@ double wxFfmpegMediaDecoder::GetPosition() {
 }
 
 wxImage wxFfmpegMediaDecoder::GetNextFrame() {
-    if (!m_frame && !BeginDecode())
-        return wxImage();
-    
-    int frameFinished;
-    AVPacket packet;
-    while (av_read_frame(m_formatCtx, &packet) >=0)
-    {
-      // is this a packet from the video stream?
-      if(packet.stream_index == m_videoStream)
-      {
-        // decode video frame
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 35, 0)
-        avcodec_decode_video2(m_codecCtx, m_frame, &frameFinished, &packet);
-#else
-		avcodec_decode_video(m_codecCtx, m_frame, &frameFinished, packet.data, packet.size);
-#endif
-        if (frameFinished)
-        {
-            SwsContext* imgConvertCtx = sws_getContext(m_codecCtx->width, m_codecCtx->height,
-                m_codecCtx->pix_fmt, m_width, m_height, PIX_FMT_RGB24, SWS_BICUBIC, NULL, NULL, NULL);
-            if (imgConvertCtx == NULL)
-                return wxImage();
-            
-            wxImage img(m_width, m_height);
-            uint8_t *rgbSrc[3]= {img.GetData(), NULL, NULL};
-            int rgbStride[3]={3*m_width, 0, 0}; 
-            sws_scale(imgConvertCtx, m_frame->data, m_frame->linesize,
-                0, m_codecCtx->height, rgbSrc, rgbStride);
-            av_free_packet(&packet);
-            return img;
-        }
-      }
-      // free the packet that was allocated by av_read_frame
-      av_free_packet(&packet);
-    }
-    return wxImage();
+	if (!m_frame && !BeginDecode())
+		return wxImage();
+
+	int frameFinished;
+	AVPacket packet;
+	 while (av_read_frame(m_formatCtx, &packet) >=0) {
+		// is this a packet from the video stream?
+		if (packet.stream_index == m_videoStream) {
+			// decode video frame
+			avcodec_decode_video2(m_codecCtx, m_frame, &frameFinished, &packet);
+			if (frameFinished) {
+				SwsContext* imgConvertCtx = sws_getContext(m_codecCtx->width, m_codecCtx->height, m_codecCtx->pix_fmt,
+						m_width, m_height, PIX_FMT_RGB24, SWS_BICUBIC, NULL, NULL, NULL);
+				if (imgConvertCtx == NULL)
+					return wxImage();
+
+				wxImage img(m_width, m_height);
+				uint8_t *rgbSrc[3] = { img.GetData(), NULL, NULL };
+				int rgbStride[3] = { 3 * m_width, 0, 0 };
+				sws_scale(imgConvertCtx, m_frame->data, m_frame->linesize, 0, m_codecCtx->height, rgbSrc, rgbStride);
+				av_free_packet(&packet);
+				return img;
+			}
+		}
+		// free the packet that was allocated by av_read_frame
+		av_free_packet(&packet);
+	}
+	return wxImage();
 }
 
 void wxFfmpegMediaDecoder::EndDecode()
